@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 /* TODO: Some plugins cause latency, and some even report it by using a control
@@ -52,7 +50,7 @@
 PA_MODULE_AUTHOR("Lennart Poettering");
 PA_MODULE_DESCRIPTION(_("Virtual LADSPA sink"));
 PA_MODULE_VERSION(PACKAGE_VERSION);
-PA_MODULE_LOAD_ONCE(FALSE);
+PA_MODULE_LOAD_ONCE(false);
 PA_MODULE_USAGE(
     _("sink_name=<name for the sink> "
       "sink_properties=<properties for the sink> "
@@ -92,7 +90,7 @@ struct userdata {
 
     pa_memblockq *memblockq;
 
-    pa_bool_t *use_default;
+    bool *use_default;
     pa_sample_spec ss;
 
 #ifdef HAVE_DBUS
@@ -100,7 +98,7 @@ struct userdata {
     char *dbus_path;
 #endif
 
-    pa_bool_t auto_desc;
+    bool auto_desc;
 };
 
 static const char* const valid_modargs[] = {
@@ -124,7 +122,7 @@ enum {
    LADSPA_SINK_MESSAGE_UPDATE_PARAMETERS = PA_SINK_MESSAGE_MAX
 };
 
-static int write_control_parameters(struct userdata *u, double *control_values, pa_bool_t *use_default);
+static int write_control_parameters(struct userdata *u, double *control_values, bool *use_default);
 static void connect_control_ports(struct userdata *u);
 
 #ifdef HAVE_DBUS
@@ -184,7 +182,7 @@ static void set_algorithm_parameters(DBusConnection *conn, DBusMessage *msg, DBu
     unsigned n_dbus_control, n_dbus_use_default;
     double *read_values = NULL;
     dbus_bool_t *read_defaults = NULL;
-    pa_bool_t *use_defaults = NULL;
+    bool *use_defaults = NULL;
     unsigned long i;
 
     pa_assert(conn);
@@ -216,7 +214,7 @@ static void set_algorithm_parameters(DBusConnection *conn, DBusMessage *msg, DBu
         return;
     }
 
-    use_defaults = pa_xnew(pa_bool_t, n_control);
+    use_defaults = pa_xnew(bool, n_control);
     for (i = 0; i < u->n_control; i++)
         use_defaults[i] = read_defaults[i];
 
@@ -403,7 +401,7 @@ static void sink_request_rewind_cb(pa_sink *s) {
     /* Just hand this one over to the master sink */
     pa_sink_input_request_rewind(u->sink_input,
                                  s->thread_info.rewind_nbytes +
-                                 pa_memblockq_get_length(u->memblockq), TRUE, FALSE, FALSE);
+                                 pa_memblockq_get_length(u->memblockq), true, false, false);
 }
 
 /* Called from I/O thread context */
@@ -511,7 +509,7 @@ static void sink_input_process_rewind_cb(pa_sink_input *i, size_t nbytes) {
         if (amount > 0) {
             unsigned c;
 
-            pa_memblockq_seek(u->memblockq, - (int64_t) amount, PA_SEEK_RELATIVE, TRUE);
+            pa_memblockq_seek(u->memblockq, - (int64_t) amount, PA_SEEK_RELATIVE, true);
 
             pa_log_debug("Resetting plugin");
 
@@ -622,7 +620,7 @@ static void sink_input_kill_cb(pa_sink_input *i) {
     pa_sink_unref(u->sink);
     u->sink = NULL;
 
-    pa_module_unload_request(u->module, TRUE);
+    pa_module_unload_request(u->module, true);
 }
 
 /* Called from IO thread context */
@@ -637,7 +635,7 @@ static void sink_input_state_change_cb(pa_sink_input *i, pa_sink_input_state_t s
     if (PA_SINK_INPUT_IS_LINKED(state) &&
             i->thread_info.state == PA_SINK_INPUT_INIT) {
         pa_log_debug("Requesting rewind due to state change.");
-        pa_sink_input_request_rewind(i, 0, FALSE, TRUE, TRUE);
+        pa_sink_input_request_rewind(i, 0, false, true, true);
     }
 }
 
@@ -678,7 +676,7 @@ static void sink_input_mute_changed_cb(pa_sink_input *i) {
     pa_sink_mute_changed(u->sink, i->muted);
 }
 
-static int parse_control_parameters(struct userdata *u, const char *cdata, double *read_values, pa_bool_t *use_default) {
+static int parse_control_parameters(struct userdata *u, const char *cdata, double *read_values, bool *use_default) {
     unsigned long p = 0;
     const char *state = NULL;
     char *k;
@@ -699,7 +697,7 @@ static int parse_control_parameters(struct userdata *u, const char *cdata, doubl
 
         if (*k == 0) {
             pa_log_debug("Read empty config value (p=%lu)", p);
-            use_default[p++] = TRUE;
+            use_default[p++] = true;
             pa_xfree(k);
             continue;
         }
@@ -714,7 +712,7 @@ static int parse_control_parameters(struct userdata *u, const char *cdata, doubl
 
         pa_log_debug("Read config value %f (p=%lu)", f, p);
 
-        use_default[p] = FALSE;
+        use_default[p] = false;
         read_values[p++] = f;
     }
 
@@ -722,7 +720,7 @@ static int parse_control_parameters(struct userdata *u, const char *cdata, doubl
        if it is left empty, so we do it here. */
     if (*cdata == 0 || cdata[strlen(cdata) - 1] == ',') {
         if (p < u->n_control)
-            use_default[p] = TRUE;
+            use_default[p] = true;
         p++;
     }
 
@@ -771,7 +769,7 @@ static void connect_control_ports(struct userdata *u) {
     }
 }
 
-static int validate_control_parameters(struct userdata *u, double *control_values, pa_bool_t *use_default) {
+static int validate_control_parameters(struct userdata *u, double *control_values, bool *use_default) {
     unsigned long p = 0, h = 0;
     const LADSPA_Descriptor *d;
     pa_sample_spec ss;
@@ -834,7 +832,7 @@ static int validate_control_parameters(struct userdata *u, double *control_value
     return 0;
 }
 
-static int write_control_parameters(struct userdata *u, double *control_values, pa_bool_t *use_default) {
+static int write_control_parameters(struct userdata *u, double *control_values, bool *use_default) {
     unsigned long p = 0, h = 0, c;
     const LADSPA_Descriptor *d;
     pa_sample_spec ss;
@@ -944,7 +942,6 @@ static int write_control_parameters(struct userdata *u, double *control_values,
     return 0;
 }
 
-
 int pa__init(pa_module*m) {
     struct userdata *u;
     pa_sample_spec ss;
@@ -960,6 +957,7 @@ int pa__init(pa_module*m) {
     const char *e, *cdata;
     const LADSPA_Descriptor *d;
     unsigned long p, h, j, n_control, c;
+    pa_memchunk silence;
 
     pa_assert(m);
 
@@ -1004,7 +1002,6 @@ int pa__init(pa_module*m) {
     u = pa_xnew0(struct userdata, 1);
     u->module = m;
     m->userdata = u;
-    u->memblockq = pa_memblockq_new("module-ladspa-sink memblockq", 0, MEMBLOCKQ_MAXLENGTH, 0, &ss, 1, 1, 0, NULL);
     u->max_ladspaport_count = 1; /*to avoid division by zero etc. in pa__done when failing before this value has been set*/
     u->channels = 0;
     u->input = NULL;
@@ -1145,7 +1142,6 @@ int pa__init(pa_module*m) {
         }
     }
 
-
     u->block_size = pa_frame_align(pa_mempool_block_size_max(m->core->mempool), &ss);
 
     /* Create buffers */
@@ -1179,15 +1175,15 @@ int pa__init(pa_module*m) {
 
     if (u->n_control > 0) {
         double *control_values;
-        pa_bool_t *use_default;
+        bool *use_default;
 
         /* temporary storage for parser */
         control_values = pa_xnew(double, (unsigned) u->n_control);
-        use_default = pa_xnew(pa_bool_t, (unsigned) u->n_control);
+        use_default = pa_xnew(bool, (unsigned) u->n_control);
 
         /* real storage */
         u->control = pa_xnew(LADSPA_Data, (unsigned) u->n_control);
-        u->use_default = pa_xnew(pa_bool_t, (unsigned) u->n_control);
+        u->use_default = pa_xnew(bool, (unsigned) u->n_control);
 
         if ((parse_control_parameters(u, cdata, control_values, use_default) < 0) ||
             (write_control_parameters(u, control_values, use_default) < 0)) {
@@ -1259,7 +1255,7 @@ int pa__init(pa_module*m) {
     pa_sink_input_new_data_init(&sink_input_data);
     sink_input_data.driver = __FILE__;
     sink_input_data.module = m;
-    pa_sink_input_new_data_set_sink(&sink_input_data, master, FALSE);
+    pa_sink_input_new_data_set_sink(&sink_input_data, master, false);
     sink_input_data.origin_sink = u->sink;
     pa_proplist_sets(sink_input_data.proplist, PA_PROP_MEDIA_NAME, "LADSPA Stream");
     pa_proplist_sets(sink_input_data.proplist, PA_PROP_MEDIA_ROLE, "filter");
@@ -1288,6 +1284,10 @@ int pa__init(pa_module*m) {
 
     u->sink->input_to_master = u->sink_input;
 
+    pa_sink_input_get_silence(u->sink_input, &silence);
+    u->memblockq = pa_memblockq_new("module-ladspa-sink memblockq", 0, MEMBLOCKQ_MAXLENGTH, 0, &ss, 1, 1, 0, &silence);
+    pa_memblock_unref(silence.memblock);
+
     pa_sink_put(u->sink);
     pa_sink_input_put(u->sink_input);
 
