@@ -17,13 +17,10 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
-typedef struct pa_card pa_card;
-
+#include <pulsecore/typedefs.h>
 #include <pulse/proplist.h>
 #include <pulsecore/core.h>
 #include <pulsecore/module.h>
@@ -37,11 +34,19 @@ typedef enum pa_available {
     PA_AVAILABLE_YES = 2,
 } pa_available_t;
 
-typedef struct pa_card_profile {
+struct pa_card_profile {
     pa_card *card;
     char *name;
     char *description;
 
+    /* Identifiers for the profile's input and output parts, i e, if two different profiles
+       have the same input_name string, they have the same source(s).
+       Same for output_name and sink(s).
+       Can be NULL (and in case of an input- or output- only profile, the other direction
+       will be NULL). */
+    char *input_name;
+    char *output_name;
+
     unsigned priority;
     pa_available_t available; /* PA_AVAILABLE_UNKNOWN, PA_AVAILABLE_NO or PA_AVAILABLE_YES */
 
@@ -53,7 +58,7 @@ typedef struct pa_card_profile {
     unsigned max_source_channels;
 
     /* .. followed by some implementation specific data */
-} pa_card_profile;
+};
 
 #define PA_CARD_PROFILE_DATA(d) ((void*) ((uint8_t*) d + PA_ALIGN(sizeof(pa_card_profile))))
 
@@ -75,7 +80,7 @@ struct pa_card {
 
     pa_hashmap *ports;
 
-    pa_bool_t save_profile:1;
+    bool save_profile:1;
 
     void *userdata;
 
@@ -94,9 +99,9 @@ typedef struct pa_card_new_data {
 
     pa_hashmap *ports;
 
-    pa_bool_t namereg_fail:1;
+    bool namereg_fail:1;
 
-    pa_bool_t save_profile:1;
+    bool save_profile:1;
 } pa_card_new_data;
 
 pa_card_profile *pa_card_profile_new(const char *name, const char *description, size_t extra);
@@ -115,8 +120,8 @@ void pa_card_free(pa_card *c);
 
 void pa_card_add_profile(pa_card *c, pa_card_profile *profile);
 
-int pa_card_set_profile(pa_card *c, const char *name, pa_bool_t save);
+int pa_card_set_profile(pa_card *c, pa_card_profile *profile, bool save);
 
-int pa_card_suspend(pa_card *c, pa_bool_t suspend, pa_suspend_cause_t cause);
+int pa_card_suspend(pa_card *c, bool suspend, pa_suspend_cause_t cause);
 
 #endif
