@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -70,7 +68,7 @@ char *pa_module_list_to_string(pa_core *c) {
         pa_xfree(t);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_client_list_to_string(pa_core *c) {
@@ -100,7 +98,7 @@ char *pa_client_list_to_string(pa_core *c) {
         pa_xfree(t);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 static const char *available_to_string(pa_available_t a) {
@@ -195,7 +193,7 @@ char *pa_card_list_to_string(pa_core *c) {
         append_port_list(s, card->ports);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 static const char *sink_state_to_string(pa_sink_state_t state) {
@@ -347,7 +345,7 @@ char *pa_sink_list_to_string(pa_core *c) {
                     sink->active_port->name);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_source_list_to_string(pa_core *c) {
@@ -462,7 +460,7 @@ char *pa_source_list_to_string(pa_core *c) {
                     source->active_port->name);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_source_output_list_to_string(pa_core *c) {
@@ -536,7 +534,7 @@ char *pa_source_output_list_to_string(pa_core *c) {
             state_table[pa_source_output_get_state(o)],
             o->source->index, o->source->name,
             volume_str,
-            pa_yes_no(pa_source_output_get_mute(o)),
+            pa_yes_no(o->muted),
             (double) pa_source_output_get_latency(o, NULL) / PA_USEC_PER_MSEC,
             clt,
             pa_sample_spec_snprint(ss, sizeof(ss), &o->sample_spec),
@@ -559,7 +557,7 @@ char *pa_source_output_list_to_string(pa_core *c) {
         pa_xfree(t);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_sink_input_list_to_string(pa_core *c) {
@@ -634,7 +632,7 @@ char *pa_sink_input_list_to_string(pa_core *c) {
             state_table[pa_sink_input_get_state(i)],
             i->sink->index, i->sink->name,
             volume_str,
-            pa_yes_no(pa_sink_input_get_mute(i)),
+            pa_yes_no(i->muted),
             (double) pa_sink_input_get_latency(i, NULL) / PA_USEC_PER_MSEC,
             clt,
             pa_sample_spec_snprint(ss, sizeof(ss), &i->sample_spec),
@@ -655,7 +653,7 @@ char *pa_sink_input_list_to_string(pa_core *c) {
         pa_xfree(t);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_scache_list_to_string(pa_core *c) {
@@ -714,7 +712,7 @@ char *pa_scache_list_to_string(pa_core *c) {
         }
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
 
 char *pa_full_status_string(pa_core *c) {
@@ -757,5 +755,5 @@ char *pa_full_status_string(pa_core *c) {
         pa_xfree(t);
     }
 
-    return pa_strbuf_tostring_free(s);
+    return pa_strbuf_to_string_free(s);
 }
