@@ -14,9 +14,7 @@
   Lesser General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  License along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -110,8 +108,8 @@ pa_memchunk* pa_memchunk_memcpy(pa_memchunk *dst, pa_memchunk *src) {
     return dst;
 }
 
-pa_bool_t pa_memchunk_isset(pa_memchunk *chunk) {
-    assert(chunk);
+bool pa_memchunk_isset(pa_memchunk *chunk) {
+    pa_assert(chunk);
 
     return
         chunk->memblock ||
